#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['mundi', 'mundi.plugin', 'mundi.types']

package_data = \
{'': ['*'], 'mundi': ['databases/*']}

install_requires = \
['pandas', 'sidekick']

extras_require = \
{'dev': ['black', 'twine', 'pycountry', 'coverage~=5.0', 'pytest~=5.0']}

setup(name='mundi',
      version='0.2.1',
      description='Centralize data from countries and regions around the World and expose it',
      author='Fábio Macêdo Mendes',
      author_email='fabiomacedomendes@gmail.com',
      url='https://github.com/pydemic/mundi',
      packages=packages,
      package_data=package_data,
      install_requires=install_requires,
      extras_require=extras_require,
     )
